@@ -3,7 +3,7 @@ use warnings;
 
 package Math::Polygon;
 use vars '$VERSION';
-$VERSION = '0.002';
+$VERSION = '0.003';
 
 use Math::Polygon::Calc;
 use Math::Polygon::Clip;
@@ -139,6 +139,12 @@ sub same($;@)
 }
 
 
+sub contains($)
+{   my ($self, $point) = @_;
+    polygon_contains_point($point, shift->points);
+}
+
+
 sub resize(@)
 {   my $self = shift;
 
